@@ -29,8 +29,7 @@ from docutils.parsers.rst.states import Body
 
 try:
     collections.Callable = collections.abc.Callable
-    from sphinxcontrib.napoleon.docstring import (
-        GoogleDocstring, NumpyDocstring)
+    from sphinxcontrib.napoleon import Config, GoogleDocstring, NumpyDocstring
 finally:
     if sys.version_info >= (3, 7):
         del collections.Callable
@@ -41,6 +40,9 @@ except ImportError:
     from typing_extensions import Literal
 
 try:
+    # colorama is a dependency on Windows to support ANSI escapes (from rst
+    # markup).  It is optional on Unices, but can still useful be there as it
+    # strips out ANSI escapes when the output is piped.
     from colorama import colorama_text as _colorama_text
 except ImportError:
     @contextlib.contextmanager
@@ -60,17 +62,15 @@ _TYPE_NAMES = ['type', 'kwtype']
 _Doc = namedtuple('_Doc', ('first_line', 'text', 'params', 'raises'))
 _Param = namedtuple('_Param', ('text', 'type'))
 
-_SUPPRESS_BOOL_DEFAULT = object()
-
 
 if hasattr(typing, 'get_args'):
     _ti_get_args = typing.get_args
 else:
     def _ti_get_args(tp):
         import typing_inspect as ti
-        if type(tp) is type(Literal):  # Py<=3.6.
+        if type(tp) is type(Literal):  # Py<3.7.
             return tp.__values__
-        return ti.get_args(tp, evaluate=True)  # evaluate=True default on Py>=3.7.
+        return ti.get_args(tp, evaluate=True)  # =True is default on Py>=3.7.
 
 
 if hasattr(typing, 'get_origin'):
@@ -78,10 +78,10 @@ if hasattr(typing, 'get_origin'):
 else:
     def _ti_get_origin(tp):
         import typing_inspect as ti
-        if type(tp) is type(Literal):  # Py<=3.6.
+        if type(tp) is type(Literal):  # Py<3.7.
             return Literal
         origin = ti.get_origin(tp)
-        return {  # Py<=3.6.
+        return {  # Py<3.7.
             typing.List: list,
             typing.Iterable: collections.abc.Iterable,
             typing.Sequence: collections.abc.Sequence,
@@ -89,6 +89,38 @@ else:
         }.get(origin, origin)
 
 
+# Modified from Py3.9's version, plus:
+# - a fix to bpo#38956 (by omitting the extraneous help string),
+# - support for short aliases for --no-foo, by moving negative flag generation
+#   to _add_argument (where the negative aliases are available),
+# - a hack (_CustomString) to simulate format_usage on Py<3.9 (_CustomString
+#   relies on an Py<3.9 implementation detail: the usage string is built using
+#   '%s' % option_strings[0] (so there is an additional call to str()) whereas
+#   the option invocation help directly joins the strings).
+
+
+class _CustomString(str):
+    def __str__(self):
+        return self.action.format_usage()
+
+
+class _BooleanOptionalAction(Action):
+    def __init__(self, option_strings, **kwargs):
+        self.negative_option_strings = []  # set by _add_argument
+        if option_strings:
+            cs = option_strings[0] = _CustomString(option_strings[0])
+            cs.action = self
+        super().__init__(option_strings, nargs=0, **kwargs)
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        if option_string in self.option_strings:
+            setattr(namespace, self.dest,
+                    option_string not in self.negative_option_strings)
+
+    def format_usage(self):
+        return ' | '.join(self.option_strings)
+
+
 class _DefaultList(list):
     """
     Marker type used to determine that a parameter corresponds to a varargs,
@@ -103,6 +135,7 @@ def run(funcs: Union[Callable, List[Callable], Dict[str, Callable]], *,
         strict_kwonly: bool = True,
         show_defaults: bool = True,
         show_types: bool = False,
+        no_negated_flags: bool = False,
         version: Union[str, None, bool] = None,
         argparse_kwargs: dict = {},
         argv: Optional[List[str]] = None):
@@ -135,6 +168,10 @@ def run(funcs: Union[Callable, List[Callable], Dict[str, Callable]], *,
         Whether parameter defaults are appneded to parameter descriptions.
     :param show_types:
         Whether parameter types are appended to parameter descriptions.
+    :param no_negated_flags:
+        If `False` (default), for any non-positional bool options, two flags
+        are created: ``--foo`` and ``--no-foo``. If `True`, the ``--no-foo``
+        is not created for every such option that has a default value `False`.
     :param version:
         If a string, add a ``--version`` flag which prints the given version
         string and exits.
@@ -156,8 +193,9 @@ def run(funcs: Union[Callable, List[Callable], Dict[str, Callable]], *,
     """
     parser = _create_parser(
         funcs, parsers=parsers, short=short, strict_kwonly=strict_kwonly,
-        show_defaults=show_defaults, show_types=show_types, version=version,
-        argparse_kwargs=argparse_kwargs)
+        show_defaults=show_defaults, show_types=show_types,
+        no_negated_flags=no_negated_flags,
+        version=version, argparse_kwargs=argparse_kwargs)
     with _colorama_text():
         args = parser.parse_args(argv)
     # Workaround for http://bugs.python.org/issue9253#msg186387
@@ -176,14 +214,15 @@ def _create_parser(
         strict_kwonly=True,
         show_defaults=True,
         show_types=False,
+        no_negated_flags=False,
         version=None,
         argparse_kwargs={}):
     parser = ArgumentParser(
         **{**{'formatter_class': RawTextHelpFormatter}, **argparse_kwargs})
     version_sources = []
     if callable(funcs):
-        _populate_parser(funcs, parser, parsers, short,
-                         strict_kwonly, show_defaults, show_types)
+        _populate_parser(funcs, parser, parsers, short, strict_kwonly,
+                         show_defaults, show_types, no_negated_flags)
         version_sources.append(funcs)
     else:
         subparsers = parser.add_subparsers()
@@ -196,8 +235,8 @@ def _create_parser(
                 name,
                 formatter_class=RawTextHelpFormatter,
                 help=_parse_docstring(inspect.getdoc(func)).first_line)
-            _populate_parser(func, subparser, parsers, short,
-                             strict_kwonly, show_defaults, show_types)
+            _populate_parser(func, subparser, parsers, short, strict_kwonly,
+                             show_defaults, show_types, no_negated_flags)
             version_sources.append(func)
     if isinstance(version, str):
         version_string = version
@@ -285,8 +324,8 @@ def signature(func: Callable):
     return full_sig.replace(parameters=parameters)
 
 
-def _populate_parser(func, parser, parsers, short,
-                     strict_kwonly, show_defaults, show_types):
+def _populate_parser(func, parser, parsers, short, strict_kwonly,
+                     show_defaults, show_types, no_negated_flags):
     sig = signature(func)
     doc = _parse_docstring(inspect.getdoc(func))
     parser.description = doc.text
@@ -318,21 +357,22 @@ def _populate_parser(func, parser, parsers, short,
         required = not hasdefault and param.kind != param.VAR_POSITIONAL
         positional = name in positionals
         if type_ == bool and not positional:
-            # Special case: just add parameterless --name and --no-name flags.
-            if default == SUPPRESS:
-                # Work around lack of "required non-exclusive group" in
-                # argparse by checking for that case ourselves.
-                default = _SUPPRESS_BOOL_DEFAULT
-            actions.extend([
-                _add_argument(parser, name, short,
-                              action='store_true',
-                              default=default,
-                              # Add help if available.
-                              **kwargs),
-                _add_argument(parser, 'no-' + name, short,
-                              action='store_false',
-                              default=default,
-                              dest=name)])
+            if no_negated_flags and default is False:
+                actions.append(
+                    _add_argument(parser, name, short,
+                                  action='store_true',
+                                  default=default,
+                                  # Add help if available.
+                                  **kwargs))
+            else:
+                # Special case:
+                # just add parameterless --name and --no-name flags.
+                actions.append(_add_argument(parser, name, short,
+                                             action=_BooleanOptionalAction,
+                                             default=default,
+                                             required=required,
+                                             # Add help if available.
+                                             **kwargs))
             continue
         # Always set a default, even for required parameters, so that we can
         # later (ab)use default == SUPPRESS (!= None) to detect required
@@ -360,12 +400,11 @@ def _populate_parser(func, parser, parsers, short,
             kwargs['action'] = _make_store_tuple_action_class(
                 tuple, member_types, parsers)
         elif (isinstance(type_, type) and issubclass(type_, tuple)
-              and hasattr(type_, '_fields')
-              and hasattr(type_, '_field_types')):
+              and hasattr(type_, '_fields')):
             # Before Py3.6, `_field_types` does not preserve order, so retrieve
             # the order from `_fields`.
-            member_types = tuple(type_._field_types[field]
-                                 for field in type_._fields)
+            hints = typing.get_type_hints(type_)
+            member_types = tuple(hints[field] for field in type_._fields)
             kwargs['nargs'] = len(member_types)
             kwargs['action'] = _make_store_tuple_action_class(
                 lambda args, type_=type_: type_(*args),
@@ -388,6 +427,7 @@ def _populate_parser(func, parser, parsers, short,
 
 
 def _add_argument(parser, name, short, _positional=False, **kwargs):
+    negative_option_strings = []
     if _positional:
         args = [name]
     else:
@@ -396,7 +436,17 @@ def _add_argument(parser, name, short, _positional=False, **kwargs):
         args = [prefix_char * 2 + name]
         if name in short:
             args.insert(0, prefix_char + short[name])
-    return parser.add_argument(*args, **kwargs)
+        if kwargs.get('action') == _BooleanOptionalAction:
+            no_name = 'no-' + name
+            if no_name in short:
+                args.append(prefix_char + short[no_name])
+                negative_option_strings.append(args[-1])
+            args.append(prefix_char * 2 + no_name)
+            negative_option_strings.append(args[-1])
+    action = parser.add_argument(*args, **kwargs)
+    if negative_option_strings:
+        action.negative_option_strings = negative_option_strings
+    return action
 
 
 def _update_help_string(action, *, show_defaults, show_types):
@@ -412,8 +462,7 @@ def _update_help_string(action, *, show_defaults, show_types):
             and action.const is not False  # i.e. action='store_false'.
             and not isinstance(action.default, _DefaultList)
             and '%(default)' not in action_help
-            and action.default is not SUPPRESS
-            and action.default is not _SUPPRESS_BOOL_DEFAULT):
+            and action.default is not SUPPRESS):
         info.append(
             'default: {}'.format(action.default.name.replace('%', '%%'))
             if action.type is not None
@@ -482,18 +531,14 @@ def _get_type_from_hint(hint):
         [type_] = _ti_get_args(hint)
         return List[type_]
     elif _ti_get_origin(hint) is Union:
-        # Flatten Union[type, NoneType] (== Optional[type]) to type.
-        # get_type_hints also appends NoneType to unions for parameters
-        # defaulting to None.
-        args = [arg for arg in _ti_get_args(hint) if arg is not type(None)]
-        if len(args) > 1:
-            if any(_is_list_like(subtype) for subtype in args):
+        args = _ti_get_args(hint)
+        if any(_is_list_like(subtype) for subtype in args):
+            non_none = [arg for arg in args if arg is not type(None)]
+            if len(non_none) != 1:
                 raise ValueError(
                     'unsupported union including container type: {}'
                     .format(hint))
-            return Union[tuple(args)]
-        else:
-            return args[0]
+            return non_none[0]
     return hint
 
 
@@ -503,9 +548,6 @@ def _call_function(parser, func, args):
     sig = signature(func)
     for name, param in sig.parameters.items():
         arg = getattr(args, name)
-        if arg is _SUPPRESS_BOOL_DEFAULT:
-            parser.error('one of the arguments --{0} --no-{0} is required'
-                         .format(name))
         if param.kind in [param.POSITIONAL_ONLY, param.POSITIONAL_OR_KEYWORD]:
             positionals.append(arg)
         elif param.kind == param.VAR_POSITIONAL:
@@ -546,8 +588,11 @@ def _parse_docstring(doc):
 
     # Convert Google- or Numpy-style docstrings to RST.
     # (Should do nothing if not in either style.)
-    doc = str(GoogleDocstring(doc))
-    doc = str(NumpyDocstring(doc))
+    # use_ivar avoids generating an unhandled .. attribute:: directive for
+    # Attribute blocks, preferring a benign :ivar: field.
+    cfg = Config(napoleon_use_ivar=True)
+    doc = str(GoogleDocstring(doc, cfg))
+    doc = str(NumpyDocstring(doc, cfg))
 
     with _sphinx_common_roles():
         tree = docutils.core.publish_doctree(doc)
@@ -563,7 +608,7 @@ def _parse_docstring(doc):
         ]
 
         def __init__(self, document):
-            NodeVisitor.__init__(self, document)
+            super().__init__(document)
             self.paragraphs = []
             self.start_lines = []
             self.params = defaultdict(dict)
@@ -736,6 +781,8 @@ def _get_parser(type_, parsers):
             parser = functools.partial(_parse_bool)
         elif type_ == slice:
             parser = functools.partial(_parse_slice)
+        elif type_ == type(None):
+            parser = functools.partial(_parse_none)
         elif type_ == list:
             raise ValueError('unable to parse list (try list[type])')
         elif isinstance(type_, type) and issubclass(type_, Enum):
@@ -789,6 +836,10 @@ def _parse_slice(string):
     return sl
 
 
+def _parse_none(string):
+    raise ValueError('No string can be converted to None')
+
+
 def _make_enum_parser(enum, value=None):
     if value is None:
         return functools.partial(_make_enum_parser, enum)
@@ -826,7 +877,7 @@ def _make_union_parser(union, parsers, value=None):
     for p in parsers:
         try:
             return p(value)
-        except (ValueError, ArgumentTypeError):
+        except (ValueError, ArgumentTypeError) as e:
             pass
     raise ValueError(
         '{} could not be parsed as any of {}'.format(value, union))
