@@ -23,7 +23,15 @@ Choice = Enum('Choice', [('one', 1), ('two', 2), ('%', 0.01)])
 Pair = typing.NamedTuple('Pair', [('first', int), ('second', str)])
 
 
+# Also check that the Attributes section doesn't trip docutils.
 class ConstructibleFromStr:
+    """
+    Attributes
+    ----------
+    s : str
+        The s.
+    """
+
     def __init__(self, s):
         """:type s: str"""
         self.s = s
@@ -177,6 +185,20 @@ class TestParsers(unittest.TestCase):
         self.assertEqual(
             defopt.run(main, parsers={int: parser}, argv=['1']), 2)
 
+    def test_overridden_none_parser(self):
+        def parser(string):
+            if string == ':none:':
+                return None
+            else:
+                raise ValueError("Not :none:")
+
+        def main(value):
+            """:type value: typing.Optional[int]"""
+            return value
+        self.assertEqual(
+            defopt.run(main, parsers={type(None): parser}, argv=[':none:']),
+            None)
+
     def test_parse_bool(self):
         parser = defopt._get_parser(bool, {})
         self.assertEqual(parser('t'), True)
@@ -185,7 +207,6 @@ class TestParsers(unittest.TestCase):
         with self.assertRaises(ValueError):
             parser('foo')
 
-    @unittest.skipIf(Path is None, 'pathlib not installed')
     def test_parse_path(self):
         def main(value):
             """:type value: Path"""
@@ -721,7 +742,9 @@ class TestAnnotations(unittest.TestCase):
                          typing.List[int])
 
     def test_optional(self):
-        self.assertEqual(defopt._get_type_from_hint(typing.Optional[int]), int)
+        self.assertEqual(
+            defopt._get_type_from_hint(typing.Optional[typing.List[int]]),
+            typing.List[int])
 
     def test_conflicting(self):
         def foo(bar: int):
@@ -783,6 +806,15 @@ class TestHelp(unittest.TestCase):
         self.assert_in_help('(default: False)', foo, 'd')
         self.assert_not_in_help('(', foo, '')
 
+    def test_no_negated_flags(self):
+        def main(*, foo=False, bar=True):
+            """
+            :param bool foo: Foo
+            :param bool bar: Bar
+            """
+        self.assert_not_in_help('--no-foo', main, 'n')
+        self.assert_in_help('--no-bar', main, 'n')
+
     def test_keyword_only(self):
         def foo(*, bar):
             """:param int bar: baz"""
@@ -877,10 +909,10 @@ class TestHelp(unittest.TestCase):
         self.assertNotIn(s, self._get_help(funcs, flags))
 
     def _get_help(self, funcs, flags):
-        self.assertLessEqual({*flags}, {'d', 't'})
+        self.assertLessEqual({*flags}, {'d', 't', 'n'})
         parser = defopt._create_parser(
             funcs, show_defaults='d' in flags, show_types='t' in flags,
-            strict_kwonly=False)
+            no_negated_flags='n' in flags, strict_kwonly=False)
         return parser.format_help()
 
 
